@@ -4,6 +4,7 @@ extern crate lazy_static;
 #[macro_use]
 extern crate serde_derive;
 
+pub mod extract_memos;
 pub mod parse_accounts;
 pub mod parse_associated_token;
 pub mod parse_bpf_loader;
@@ -14,24 +15,26 @@ pub mod parse_token;
 pub mod parse_vote;
 pub mod token_balances;
 
-use crate::{
-    parse_accounts::{parse_accounts, ParsedAccount},
-    parse_instruction::{parse, ParsedInstruction},
+pub use {crate::extract_memos::extract_and_fmt_memos, solana_runtime::bank::RewardType};
+use {
+    crate::{
+        parse_accounts::{parse_accounts, ParsedAccount},
+        parse_instruction::{parse, ParsedInstruction},
+    },
+    safecoin_account_decoder::parse_token::UiTokenAmount,
+    safecoin_sdk::{
+        clock::{Slot, UnixTimestamp},
+        commitment_config::CommitmentConfig,
+        deserialize_utils::default_on_eof,
+        instruction::CompiledInstruction,
+        message::{Message, MessageHeader},
+        pubkey::Pubkey,
+        sanitize::Sanitize,
+        signature::Signature,
+        transaction::{Result, Transaction, TransactionError},
+    },
+    std::fmt,
 };
-use safecoin_account_decoder::parse_token::UiTokenAmount;
-pub use solana_runtime::bank::RewardType;
-use solana_sdk::{
-    clock::{Slot, UnixTimestamp},
-    commitment_config::CommitmentConfig,
-    deserialize_utils::default_on_eof,
-    instruction::CompiledInstruction,
-    message::{Message, MessageHeader},
-    pubkey::Pubkey,
-    sanitize::Sanitize,
-    signature::Signature,
-    transaction::{Result, Transaction, TransactionError},
-};
-use std::fmt;
 /// A duplicate representation of an Instruction for pretty JSON serialization
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase", untagged)]
@@ -124,6 +127,7 @@ pub struct TransactionTokenBalance {
     pub account_index: u8,
     pub mint: String,
     pub ui_token_amount: UiTokenAmount,
+    pub owner: String,
 }
 
 #[derive(Clone, Debug, PartialEq, Serialize, Deserialize)]
@@ -132,6 +136,8 @@ pub struct UiTransactionTokenBalance {
     pub account_index: u8,
     pub mint: String,
     pub ui_token_amount: UiTokenAmount,
+    #[serde(default, skip_serializing_if = "Option::is_none")]
+    pub owner: Option<String>,
 }
 
 impl From<TransactionTokenBalance> for UiTransactionTokenBalance {
@@ -140,6 +146,11 @@ impl From<TransactionTokenBalance> for UiTransactionTokenBalance {
             account_index: token_balance.account_index,
             mint: token_balance.mint,
             ui_token_amount: token_balance.ui_token_amount,
+            owner: if !token_balance.owner.is_empty() {
+                Some(token_balance.owner)
+            } else {
+                None
+            },
         }
     }
 }
@@ -210,7 +221,7 @@ impl Default for TransactionStatusMeta {
 #[serde(rename_all = "camelCase")]
 pub struct UiTransactionStatusMeta {
     pub err: Option<TransactionError>,
-    pub status: Result<()>, // This field is deprecated.  See https://github.com/solana-labs/solana/issues/9302
+    pub status: Result<()>, // This field is deprecated.  See https://github.com/fair-exchange/safecoin/issues/9302
     pub fee: u64,
     pub pre_balances: Vec<u64>,
     pub post_balances: Vec<u64>,
@@ -339,6 +350,7 @@ pub struct Reward {
     pub lamports: i64,
     pub post_balance: u64, // Account balance in lamports after `lamports` was applied
     pub reward_type: Option<RewardType>,
+    pub commission: Option<u8>, // Vote account commission when the reward was credited, only present for voting and staking rewards
 }
 
 pub type Rewards = Vec<Reward>;
