@@ -1,37 +1,43 @@
-use crate::{
-    parse_associated_token::{parse_associated_token, spl_associated_token_id_v1_0},
-    parse_bpf_loader::{parse_bpf_loader, parse_bpf_upgradeable_loader},
-    parse_stake::parse_stake,
-    parse_system::parse_system,
-    parse_token::parse_token,
-    parse_vote::parse_vote,
+use {
+    crate::{
+        extract_memos::{safe_memo_id_v1, safe_memo_id_v3},
+        parse_associated_token::{parse_associated_token, spl_associated_token_id_v1_0},
+        parse_bpf_loader::{parse_bpf_loader, parse_bpf_upgradeable_loader},
+        parse_stake::parse_stake,
+        parse_system::parse_system,
+        parse_token::parse_token,
+        parse_vote::parse_vote,
+    },
+    inflector::Inflector,
+    serde_json::Value,
+    safecoin_account_decoder::parse_token::safe_token_id_v2_0,
+    safecoin_sdk::{instruction::CompiledInstruction, pubkey::Pubkey, stake, system_program},
+    std::{
+        collections::HashMap,
+        str::{from_utf8, Utf8Error},
+    },
+    thiserror::Error,
 };
-use inflector::Inflector;
-use serde_json::Value;
-use safecoin_account_decoder::parse_token::spl_token_id_v2_0;
-use solana_sdk::{instruction::CompiledInstruction, pubkey::Pubkey, system_program};
-use std::{collections::HashMap, str::from_utf8};
-use thiserror::Error;
 
 lazy_static! {
     static ref ASSOCIATED_TOKEN_PROGRAM_ID: Pubkey = spl_associated_token_id_v1_0();
-    static ref BPF_LOADER_PROGRAM_ID: Pubkey = solana_sdk::bpf_loader::id();
-    static ref BPF_UPGRADEABLE_LOADER_PROGRAM_ID: Pubkey = solana_sdk::bpf_loader_upgradeable::id();
-    static ref MEMO_V1_PROGRAM_ID: Pubkey = Pubkey::new_from_array(safe_memo::v1::id().to_bytes());
-    static ref MEMO_V3_PROGRAM_ID: Pubkey = Pubkey::new_from_array(safe_memo::id().to_bytes());
-    static ref STAKE_PROGRAM_ID: Pubkey = solana_stake_program::id();
+    static ref BPF_LOADER_PROGRAM_ID: Pubkey = safecoin_sdk::bpf_loader::id();
+    static ref BPF_UPGRADEABLE_LOADER_PROGRAM_ID: Pubkey = safecoin_sdk::bpf_loader_upgradeable::id();
+    static ref MEMO_V1_PROGRAM_ID: Pubkey = safe_memo_id_v1();
+    static ref MEMO_V3_PROGRAM_ID: Pubkey = safe_memo_id_v3();
+    static ref STAKE_PROGRAM_ID: Pubkey = stake::program::id();
     static ref SYSTEM_PROGRAM_ID: Pubkey = system_program::id();
-    static ref TOKEN_PROGRAM_ID: Pubkey = spl_token_id_v2_0();
+    static ref TOKEN_PROGRAM_ID: Pubkey = safe_token_id_v2_0();
     static ref VOTE_PROGRAM_ID: Pubkey = solana_vote_program::id();
     static ref PARSABLE_PROGRAM_IDS: HashMap<Pubkey, ParsableProgram> = {
         let mut m = HashMap::new();
         m.insert(
             *ASSOCIATED_TOKEN_PROGRAM_ID,
-            ParsableProgram::SplAssociatedTokenAccount,
+            ParsableProgram::SafeAssociatedTokenAccount,
         );
-        m.insert(*MEMO_V1_PROGRAM_ID, ParsableProgram::SplMemo);
-        m.insert(*MEMO_V3_PROGRAM_ID, ParsableProgram::SplMemo);
-        m.insert(*TOKEN_PROGRAM_ID, ParsableProgram::SplToken);
+        m.insert(*MEMO_V1_PROGRAM_ID, ParsableProgram::SafeMemo);
+        m.insert(*MEMO_V3_PROGRAM_ID, ParsableProgram::SafeMemo);
+        m.insert(*TOKEN_PROGRAM_ID, ParsableProgram::SafeToken);
         m.insert(*BPF_LOADER_PROGRAM_ID, ParsableProgram::BpfLoader);
         m.insert(
             *BPF_UPGRADEABLE_LOADER_PROGRAM_ID,
@@ -79,9 +85,9 @@ pub struct ParsedInstructionEnum {
 #[derive(Debug, Serialize, Deserialize, PartialEq)]
 #[serde(rename_all = "camelCase")]
 pub enum ParsableProgram {
-    SplAssociatedTokenAccount,
-    SplMemo,
-    SplToken,
+    SafeAssociatedTokenAccount,
+    SafeMemo,
+    SafeToken,
     BpfLoader,
     BpfUpgradeableLoader,
     Stake,
@@ -98,11 +104,11 @@ pub fn parse(
         .get(program_id)
         .ok_or(ParseInstructionError::ProgramNotParsable)?;
     let parsed_json = match program_name {
-        ParsableProgram::SplAssociatedTokenAccount => {
+        ParsableProgram::SafeAssociatedTokenAccount => {
             serde_json::to_value(parse_associated_token(instruction, account_keys)?)?
         }
-        ParsableProgram::SplMemo => parse_memo(instruction)?,
-        ParsableProgram::SplToken => serde_json::to_value(parse_token(instruction, account_keys)?)?,
+        ParsableProgram::SafeMemo => parse_memo(instruction)?,
+        ParsableProgram::SafeToken => serde_json::to_value(parse_token(instruction, account_keys)?)?,
         ParsableProgram::BpfLoader => {
             serde_json::to_value(parse_bpf_loader(instruction, account_keys)?)?
         }
@@ -121,9 +127,13 @@ pub fn parse(
 }
 
 fn parse_memo(instruction: &CompiledInstruction) -> Result<Value, ParseInstructionError> {
-    from_utf8(&instruction.data)
-        .map(|s| Value::String(s.to_string()))
-        .map_err(|_| ParseInstructionError::InstructionNotParsable(ParsableProgram::SplMemo))
+    parse_memo_data(&instruction.data)
+        .map(Value::String)
+        .map_err(|_| ParseInstructionError::InstructionNotParsable(ParsableProgram::SafeMemo))
+}
+
+pub fn parse_memo_data(data: &[u8]) -> Result<String, Utf8Error> {
+    from_utf8(data).map(|s| s.to_string())
 }
 
 pub(crate) fn check_num_accounts(
@@ -142,8 +152,7 @@ pub(crate) fn check_num_accounts(
 
 #[cfg(test)]
 mod test {
-    use super::*;
-    use serde_json::json;
+    use {super::*, serde_json::json};
 
     #[test]
     fn test_parse() {
