@@ -1,10 +1,13 @@
-use crate::parse_instruction::{
-    check_num_accounts, ParsableProgram, ParseInstructionError, ParsedInstructionEnum,
+use {
+    crate::parse_instruction::{
+        check_num_accounts, ParsableProgram, ParseInstructionError, ParsedInstructionEnum,
+    },
+    bincode::deserialize,
+    serde_json::{json, Map},
+    safecoin_sdk::{
+        instruction::CompiledInstruction, pubkey::Pubkey, stake::instruction::StakeInstruction,
+    },
 };
-use bincode::deserialize;
-use serde_json::{json, Map};
-use solana_sdk::{instruction::CompiledInstruction, pubkey::Pubkey};
-use solana_stake_program::stake_instruction::StakeInstruction;
 
 pub fn parse_stake(
     instruction: &CompiledInstruction,
@@ -185,7 +188,6 @@ pub fn parse_stake(
                 info: value,
             })
         }
-
         StakeInstruction::InitializeChecked => {
             check_num_stake_accounts(&instruction.accounts, 4)?;
             Ok(ParsedInstructionEnum {
@@ -264,24 +266,27 @@ pub fn parse_stake(
                     "custodian": account_keys[instruction.accounts[1] as usize].to_string(),
                     "lockup": lockup_map,
                 }),
-	})
-	}
-
+            })
+        }
     }
 }
 
-
 fn check_num_stake_accounts(accounts: &[u8], num: usize) -> Result<(), ParseInstructionError> {
     check_num_accounts(accounts, num, ParsableProgram::Stake)
 }
 
 #[cfg(test)]
 mod test {
-    use super::*;
-    use solana_sdk::{message::Message, pubkey::Pubkey};
-    use solana_stake_program::{
-        stake_instruction::{self, LockupArgs},
-        stake_state::{Authorized, Lockup, StakeAuthorize},
+    use {
+        super::*,
+        safecoin_sdk::{
+            message::Message,
+            pubkey::Pubkey,
+            stake::{
+                instruction::{self, LockupArgs},
+                state::{Authorized, Lockup, StakeAuthorize},
+            },
+        },
     };
 
     #[test]
@@ -304,7 +309,7 @@ mod test {
         let lamports = 55;
 
         let instructions =
-            stake_instruction::create_account(&keys[0], &keys[1], &authorized, &lockup, lamports);
+            instruction::create_account(&keys[0], &keys[1], &authorized, &lockup, lamports);
         let message = Message::new(&instructions, None);
         assert_eq!(
             parse_stake(&message.instructions[1], &keys[0..3]).unwrap(),
@@ -327,13 +332,8 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[1], &keys[0..2]).is_err());
 
-        let instruction = stake_instruction::authorize(
-            &keys[1],
-            &keys[0],
-            &keys[3],
-            StakeAuthorize::Staker,
-            None,
-        );
+        let instruction =
+            instruction::authorize(&keys[1], &keys[0], &keys[3], StakeAuthorize::Staker, None);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
@@ -350,23 +350,23 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
 
-        let instruction = stake_instruction::authorize(
-            &keys[1],
+        let instruction = instruction::authorize(
+            &keys[2],
             &keys[0],
-            &keys[3],
+            &keys[4],
             StakeAuthorize::Withdrawer,
             Some(&keys[1]),
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorize".to_string(),
                 info: json!({
-                    "stakeAccount": keys[1].to_string(),
-                    "clockSysvar": keys[2].to_string(),
+                    "stakeAccount": keys[2].to_string(),
+                    "clockSysvar": keys[3].to_string(),
                     "authority": keys[0].to_string(),
-                    "newAuthority": keys[3].to_string(),
+                    "newAuthority": keys[4].to_string(),
                     "authorityType": StakeAuthorize::Withdrawer,
                     "custodian": keys[1].to_string(),
                 }),
@@ -374,7 +374,7 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
 
-        let instruction = stake_instruction::delegate_stake(&keys[1], &keys[0], &keys[2]);
+        let instruction = instruction::delegate_stake(&keys[1], &keys[0], &keys[2]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..6]).unwrap(),
@@ -396,7 +396,7 @@ mod test {
         //  * split account (signer, allocate + assign first)
         //  * stake authority (signer)
         //  * stake account
-        let instructions = stake_instruction::split(&keys[2], &keys[1], lamports, &keys[0]);
+        let instructions = instruction::split(&keys[2], &keys[1], lamports, &keys[0]);
         let message = Message::new(&instructions, None);
         assert_eq!(
             parse_stake(&message.instructions[2], &keys[0..3]).unwrap(),
@@ -412,7 +412,7 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[2], &keys[0..2]).is_err());
 
-        let instruction = stake_instruction::withdraw(&keys[1], &keys[0], &keys[2], lamports, None);
+        let instruction = instruction::withdraw(&keys[1], &keys[0], &keys[2], lamports, None);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
@@ -429,7 +429,7 @@ mod test {
             }
         );
         let instruction =
-            stake_instruction::withdraw(&keys[2], &keys[0], &keys[3], lamports, Some(&keys[1]));
+            instruction::withdraw(&keys[2], &keys[0], &keys[3], lamports, Some(&keys[1]));
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..6]).unwrap(),
@@ -448,7 +448,7 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
 
-        let instruction = stake_instruction::deactivate_stake(&keys[1], &keys[0]);
+        let instruction = instruction::deactivate_stake(&keys[1], &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
@@ -463,7 +463,7 @@ mod test {
         );
         assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
 
-        let instructions = stake_instruction::merge(&keys[1], &keys[0], &keys[2]);
+        let instructions = instruction::merge(&keys[1], &keys[0], &keys[2]);
         let message = Message::new(&instructions, None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
@@ -481,11 +481,11 @@ mod test {
         assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
 
         let seed = "test_seed";
-        let instruction = stake_instruction::authorize_with_seed(
+        let instruction = instruction::authorize_with_seed(
             &keys[1],
             &keys[0],
             seed.to_string(),
-            &keys[2],
+            &keys[0],
             &keys[3],
             StakeAuthorize::Staker,
             None,
@@ -497,7 +497,7 @@ mod test {
                 instruction_type: "authorizeWithSeed".to_string(),
                 info: json!({
                     "stakeAccount": keys[1].to_string(),
-                    "authorityOwner": keys[2].to_string(),
+                    "authorityOwner": keys[0].to_string(),
                     "newAuthorized": keys[3].to_string(),
                     "authorityBase": keys[0].to_string(),
                     "authoritySeed": seed,
@@ -506,26 +506,26 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
 
-        let instruction = stake_instruction::authorize_with_seed(
-            &keys[1],
+        let instruction = instruction::authorize_with_seed(
+            &keys[2],
             &keys[0],
             seed.to_string(),
-            &keys[2],
-            &keys[3],
+            &keys[0],
+            &keys[4],
             StakeAuthorize::Withdrawer,
-            Some(&keys[4]),
+            Some(&keys[1]),
         );
         let message = Message::new(&[instruction], None);
         assert_eq!(
-            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
             ParsedInstructionEnum {
                 instruction_type: "authorizeWithSeed".to_string(),
                 info: json!({
                     "stakeAccount": keys[2].to_string(),
-                    "authorityOwner": keys[2].to_string(),
-                    "newAuthorized": keys[3].to_string(),
+                    "authorityOwner": keys[0].to_string(),
+                    "newAuthorized": keys[4].to_string(),
                     "authorityBase": keys[0].to_string(),
                     "authoritySeed": seed,
                     "authorityType": StakeAuthorize::Withdrawer,
@@ -534,14 +534,14 @@ mod test {
                 }),
             }
         );
-        assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
     }
 
     #[test]
     #[allow(clippy::same_item_push)]
-    fn test_parse_set_lockup() {
+    fn test_parse_stake_set_lockup() {
         let mut keys: Vec<Pubkey> = vec![];
-        for _ in 0..2 {
+        for _ in 0..3 {
             keys.push(Pubkey::new_unique());
         }
         let unix_timestamp = 1_234_567_890;
@@ -553,7 +553,7 @@ mod test {
             epoch: None,
             custodian: None,
         };
-        let instruction = stake_instruction::set_lockup(&keys[1], &lockup, &keys[0]);
+        let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
@@ -574,7 +574,7 @@ mod test {
             epoch: Some(epoch),
             custodian: None,
         };
-        let instruction = stake_instruction::set_lockup(&keys[1], &lockup, &keys[0]);
+        let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
@@ -596,7 +596,7 @@ mod test {
             epoch: Some(epoch),
             custodian: Some(custodian),
         };
-        let instruction = stake_instruction::set_lockup(&keys[1], &lockup, &keys[0]);
+        let instruction = instruction::set_lockup(&keys[1], &lockup, &keys[0]);
         let message = Message::new(&[instruction], None);
         assert_eq!(
             parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
@@ -615,5 +615,208 @@ mod test {
         );
 
         assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+
+        let lockup = LockupArgs {
+            unix_timestamp: Some(unix_timestamp),
+            epoch: None,
+            custodian: None,
+        };
+        let instruction = instruction::set_lockup_checked(&keys[1], &lockup, &keys[0]);
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "setLockupChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[1].to_string(),
+                    "custodian": keys[0].to_string(),
+                    "lockup": {
+                        "unixTimestamp": unix_timestamp
+                    }
+                }),
+            }
+        );
+
+        let lockup = LockupArgs {
+            unix_timestamp: Some(unix_timestamp),
+            epoch: Some(epoch),
+            custodian: None,
+        };
+        let instruction = instruction::set_lockup_checked(&keys[1], &lockup, &keys[0]);
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..2]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "setLockupChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[1].to_string(),
+                    "custodian": keys[0].to_string(),
+                    "lockup": {
+                        "unixTimestamp": unix_timestamp,
+                        "epoch": epoch,
+                    }
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..1]).is_err());
+
+        let lockup = LockupArgs {
+            unix_timestamp: Some(unix_timestamp),
+            epoch: Some(epoch),
+            custodian: Some(keys[1]),
+        };
+        let instruction = instruction::set_lockup_checked(&keys[2], &lockup, &keys[0]);
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..3]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "setLockupChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[2].to_string(),
+                    "custodian": keys[0].to_string(),
+                    "lockup": {
+                        "unixTimestamp": unix_timestamp,
+                        "epoch": epoch,
+                        "custodian": keys[1].to_string(),
+                    }
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..2]).is_err());
+    }
+
+    #[test]
+    #[allow(clippy::same_item_push)]
+    fn test_parse_stake_checked_instructions() {
+        let mut keys: Vec<Pubkey> = vec![];
+        for _ in 0..6 {
+            keys.push(Pubkey::new_unique());
+        }
+
+        let authorized = Authorized {
+            staker: keys[3],
+            withdrawer: keys[0],
+        };
+        let lamports = 55;
+
+        let instructions =
+            instruction::create_account_checked(&keys[0], &keys[1], &authorized, lamports);
+        let message = Message::new(&instructions, None);
+        assert_eq!(
+            parse_stake(&message.instructions[1], &keys[0..4]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "initializeChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[1].to_string(),
+                    "rentSysvar": keys[2].to_string(),
+                    "staker": keys[3].to_string(),
+                    "withdrawer": keys[0].to_string(),
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[1], &keys[0..3]).is_err());
+
+        let instruction = instruction::authorize_checked(
+            &keys[2],
+            &keys[0],
+            &keys[1],
+            StakeAuthorize::Staker,
+            None,
+        );
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "authorizeChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[2].to_string(),
+                    "clockSysvar": keys[3].to_string(),
+                    "authority": keys[0].to_string(),
+                    "newAuthority": keys[1].to_string(),
+                    "authorityType": StakeAuthorize::Staker,
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
+
+        let instruction = instruction::authorize_checked(
+            &keys[3],
+            &keys[0],
+            &keys[1],
+            StakeAuthorize::Withdrawer,
+            Some(&keys[2]),
+        );
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "authorizeChecked".to_string(),
+                info: json!({
+                    "stakeAccount": keys[3].to_string(),
+                    "clockSysvar": keys[4].to_string(),
+                    "authority": keys[0].to_string(),
+                    "newAuthority": keys[1].to_string(),
+                    "authorityType": StakeAuthorize::Withdrawer,
+                    "custodian": keys[2].to_string(),
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
+
+        let seed = "test_seed";
+        let instruction = instruction::authorize_checked_with_seed(
+            &keys[2],
+            &keys[0],
+            seed.to_string(),
+            &keys[0],
+            &keys[1],
+            StakeAuthorize::Staker,
+            None,
+        );
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..4]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "authorizeCheckedWithSeed".to_string(),
+                info: json!({
+                    "stakeAccount": keys[2].to_string(),
+                    "authorityOwner": keys[0].to_string(),
+                    "newAuthorized": keys[1].to_string(),
+                    "authorityBase": keys[0].to_string(),
+                    "authoritySeed": seed,
+                    "authorityType": StakeAuthorize::Staker,
+                    "clockSysvar": keys[3].to_string(),
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..3]).is_err());
+
+        let instruction = instruction::authorize_checked_with_seed(
+            &keys[3],
+            &keys[0],
+            seed.to_string(),
+            &keys[0],
+            &keys[1],
+            StakeAuthorize::Withdrawer,
+            Some(&keys[2]),
+        );
+        let message = Message::new(&[instruction], None);
+        assert_eq!(
+            parse_stake(&message.instructions[0], &keys[0..5]).unwrap(),
+            ParsedInstructionEnum {
+                instruction_type: "authorizeCheckedWithSeed".to_string(),
+                info: json!({
+                    "stakeAccount": keys[3].to_string(),
+                    "authorityOwner": keys[0].to_string(),
+                    "newAuthorized": keys[1].to_string(),
+                    "authorityBase": keys[0].to_string(),
+                    "authoritySeed": seed,
+                    "authorityType": StakeAuthorize::Withdrawer,
+                    "clockSysvar": keys[4].to_string(),
+                    "custodian": keys[2].to_string(),
+                }),
+            }
+        );
+        assert!(parse_stake(&message.instructions[0], &keys[0..4]).is_err());
     }
 }
